@@ -34,7 +34,6 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <err.h>
 
 #include "array.h"
 #include "mode.h"
@@ -214,7 +213,8 @@ file_read(const struct placefile *pf, int fd, const char *name, bool toplevel)
 
 			if (result == -1) {
 				/* read error */
-				warn("%s", name);
+				complain(NULL, "%s: %s",
+					 name, strerror(errno));
 				complain_fail();
 			} else if (result == 0 && bufend == linestart) {
 				/* eof */
@@ -332,7 +332,7 @@ file_tryopen(const char *file)
 	fd = open(file, O_RDONLY);
 	if (fd < 0) {
 		if (errno != ENOENT && errno != ENOTDIR) {
-			warn("%s", file);
+			complain(NULL, "%s: %s", file, strerror(errno));
 		}
 		return -1;
 	}
@@ -406,7 +406,7 @@ file_readabsolute(struct place *place, const char *name)
 	} else {
 		fd = file_tryopen(name);
 		if (fd < 0) {
-			warn("%s", name);
+			complain(NULL, "%s: %s", name, strerror(errno));
 			die();
 		}
 		pf = place_addfile(place, name, false);
