@@ -536,7 +536,7 @@ macro_parse_expansion(struct macro *m, const char *buf)
 			wordstart = pos;
 			pos += strspn(buf+pos, alnum);
 			if (isparam(m, buf+wordstart, pos-wordstart, &param)) {
-				if (pos > blockstart) {
+				if (wordstart > blockstart) {
 					ei = expansionitem_create_stringlen(
 						buf + blockstart,
 						wordstart - blockstart);
@@ -1088,7 +1088,7 @@ doexpand(struct expstate *es, struct place *p, char *buf, size_t len)
 			x = len;
 		}
 
-		if (x > 0) {
+		if (!inquote && x > 0) {
 			expand_got_word(es, p, buf, x);
 			buf += x;
 			len -= x;
