@@ -29,10 +29,10 @@
 
 #include <stdbool.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <err.h>
 
 #include "version.h"
 #include "config.h"
@@ -82,9 +82,6 @@ struct warns warns = {
 	.unused = false,
 };
 
-/* this is always true, but can be set explicitly with -traditional */
-static bool traditional = true;
-
 ////////////////////////////////////////////////////////////
 // commandline macros
 
@@ -145,7 +142,7 @@ commandline_def(const struct place *p, char *str)
 	char *val;
 
 	if (*str == '\0') {
-		warnx("-D: macro name expected");
+		complain(NULL, "-D: macro name expected");
 		die();
 	}
 
@@ -169,7 +166,7 @@ void
 commandline_undef(const struct place *p, char *str)
 {
 	if (*str == '\0') {
-		warnx("-D: macro name expected");
+		complain(NULL, "-D: macro name expected");
 		die();
 	}
 	commandline_macro_add(p, str, p, NULL);
@@ -376,7 +373,7 @@ void
 commandline_addincpath(struct stringarray *arr, char *s)
 {
 	if (*s == '\0') {
-		warnx("Empty include path");
+		complain(NULL, "Empty include directory");
 		die();
 	}
 	stringarray_add(arr, s, NULL);
@@ -493,7 +490,7 @@ commandline_addincpath_user_withprefix(const struct place *p, char *dir)
 	char *s;
 
 	if (commandline_prefix == NULL) {
-		warnx("-iprefix needed");
+		complain(NULL, "-iprefix needed");
 		die();
 	}
 	s = dostrdup3(commandline_prefix, "/", dir);
@@ -508,7 +505,7 @@ commandline_addincpath_late_withprefix(const struct place *p, char *dir)
 	char *s;
 
 	if (commandline_prefix == NULL) {
-		warnx("-iprefix needed");
+		complain(NULL, "-iprefix needed");
 		die();
 	}
 	s = dostrdup3(commandline_prefix, "/", dir);
@@ -525,7 +522,7 @@ commandline_setstd(const struct place *p, char *std)
 	if (!strcmp(std, "krc")) {
 		return;
 	}
-	warnx("Standard %s not supported by this preprocessor", std);
+	complain(NULL, "Standard %s not supported by this preprocessor", std);
 	die();
 }
 
@@ -538,7 +535,7 @@ commandline_setlang(const struct place *p, char *lang)
 	if (!strcmp(lang, "c") || !strcmp(lang, "assembler-with-cpp")) {
 		return;
 	}
-	warnx("Language %s not supported by this preprocessor", lang);
+	complain(NULL, "Language %s not supported by this preprocessor", lang);
 	die();
 }
 
@@ -552,7 +549,7 @@ commandline_iremap(const struct place *p, char *str)
 	(void)p;
 	/* XXX */
 	(void)str;
-	warnx("-iremap not supported");
+	complain(NULL, "-iremap not supported");
 	die();
 }
 
@@ -568,18 +565,18 @@ commandline_tabstop(const struct place *p, char *s)
 	t = strchr(s, '=');
 	if (t == NULL) {
 		/* should not happen */
-		warnx("Invalid tabstop");
+		complain(NULL, "Invalid tabstop");
 		die();
 	}
 	t++;
 	errno = 0;
 	val = strtoul(t, &t, 10);
 	if (errno || *t != '\0') {
-		warnx("Invalid tabstop");
+		complain(NULL, "Invalid tabstop");
 		die();
 	}
 	if (val > 64) {
-		warnx("Preposterously large tabstop");
+		complain(NULL, "Preposterously large tabstop");
 		die();
 	}
 	mode.input_tabstop = val;
@@ -734,6 +731,10 @@ commandline_wnone(void)
 ////////////////////////////////////////////////////////////
 // options
 
+struct ignore_option {
+	const char *string;
+};
+
 struct flag_option {
 	const char *string;
 	bool *flag;
@@ -755,6 +756,12 @@ struct arg_option {
 	void (*func)(const struct place *, char *);
 };
 
+static const struct ignore_option ignore_options[] = {
+	{ "m32" },
+	{ "traditional" },
+};
+static const unsigned num_ignore_options = HOWMANY(ignore_options);
+
 static const struct flag_option flag_options[] = {
 	{ "C",                          &mode.output_retain_comments,  true },
 	{ "CC",                         &mode.output_retain_comments,  true },
@@ -774,7 +781,6 @@ static const struct flag_option flag_options[] = {
 	{ "fdollars-in-identifiers",    &mode.input_allow_dollars,     true },
 	{ "fno-dollars-in-identifiers", &mode.input_allow_dollars,     false },
 	{ "nostdinc",                   &mode.do_stdinc,               false },
-	{ "traditional",		&traditional,    	       true },
 	{ "undef",                      &mode.do_stddef,               false },
 };
 static const unsigned num_flag_options = HOWMANY(flag_options);
@@ -822,6 +828,25 @@ static const struct arg_option arg_options[] = {
 };
 static const unsigned num_arg_options = HOWMANY(arg_options);
 
+static
+bool
+check_ignore_option(const char *opt)
+{
+	unsigned i;
+	int r;
+
+	for (i=0; i<num_ignore_options; i++) {
+		r = strcmp(opt, ignore_options[i].string);
+		if (r == 0) {
+			return true;
+		}
+		if (r < 0) {
+			break;
+		}
+	}
+	return false;
+}
+
 static
 bool
 check_flag_option(const char *opt)
@@ -894,7 +919,9 @@ check_arg_option(const char *opt, const struct place *argplace, char *arg)
 		r = strcmp(opt, arg_options[i].string);
 		if (r == 0) {
 			if (arg == NULL) {
-				warnx("Option -%s requires an argument", opt);
+				complain(NULL,
+					 "Option -%s requires an argument",
+					 opt);
 				die();
 			}
 			arg_options[i].func(argplace, arg);
@@ -909,14 +936,10 @@ check_arg_option(const char *opt, const struct place *argplace, char *arg)
 
 DEAD static
 void
-usage(const char *argv0, const char *fmt, ...)
+usage(const char *progname, const char *fmt, ...)
 {
-	const char *progname;
 	va_list ap;
 
-	progname = strrchr(argv0, '/');
-	progname = progname == NULL ? argv0 : progname + 1;
-
 	fprintf(stderr, "%s: ", progname);
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
@@ -1004,11 +1027,16 @@ freestringlater(char *s)
 int
 main(int argc, char *argv[])
 {
+	const char *progname;
 	const char *inputfile = NULL;
 	const char *outputfile = NULL;
 	struct place cmdplace;
 	int i;
 
+	progname = strrchr(argv[0], '/');
+	progname = progname == NULL ? argv[0] : progname + 1;
+	complain_init(progname);
+
 	init();
 
 	for (i=1; i<argc; i++) {
@@ -1016,6 +1044,9 @@ main(int argc, char *argv[])
 			break;
 		}
 		place_setcommandline(&cmdplace, i, 1);
+		if (check_ignore_option(argv[i]+1)) {
+			continue;
+		}
 		if (check_flag_option(argv[i]+1)) {
 			continue;
 		}
@@ -1030,7 +1061,7 @@ main(int argc, char *argv[])
 			i++;
 			continue;
 		}
-		usage(argv[0], "Invalid option %s", argv[i]);
+		usage(progname, "Invalid option %s", argv[i]);
 	}
 	if (i < argc) {
 		inputfile = argv[i++];
@@ -1039,7 +1070,7 @@ main(int argc, char *argv[])
 		outputfile = argv[i++];
 	}
 	if (i < argc) {
-		usage(argv[0], "Extra non-option argument %s", argv[i]);
+		usage(progname, "Extra non-option argument %s", argv[i]);
 	}
 
 	mode.output_file = outputfile;
