@@ -50,6 +50,8 @@ DEFARRAY(placefile, static);
 static struct placefilearray placefiles;
 static bool overall_failure;
 
+static const char *myprogname;
+
 ////////////////////////////////////////////////////////////
 // seenfiles
 
@@ -186,13 +188,24 @@ place_printfrom(const struct place *p)
 ////////////////////////////////////////////////////////////
 // complaints
 
+void
+complain_init(const char *pn)
+{
+	myprogname = pn;
+}
+
 void
 complain(const struct place *p, const char *fmt, ...)
 {
 	va_list ap;
 
-	place_printfrom(p);
-	fprintf(stderr, "%s:%u:%u: ", place_getname(p), p->line, p->column);
+	if (p != NULL) {
+		place_printfrom(p);
+		fprintf(stderr, "%s:%u:%u: ", place_getname(p),
+			p->line, p->column);
+	} else {
+		fprintf(stderr, "%s: ", myprogname);
+	}
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
